@@ -65,7 +65,7 @@
 # make it effectively proprietary.  To prevent this, the GPL assures that
 # patents cannot be used to render the program non-free.
 
-# Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 2.1.0 - March 13, 2021
+# Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 2.2.3 - April 7, 2021
 # simplicity and transparency in power systems planning
 
 # Developed by
@@ -89,31 +89,31 @@ import os
 # import pkg_resources
 from openTEPES.openTEPES import openTEPES_run
 import openTEPES
-CASE = "9n"
-parser = argparse.ArgumentParser(description='Introducing main parameters.')
-parser.add_argument('--case', type=str, default=None)
-parser.add_argument('--dir', type=str, default=None)
+
+CASE = '9n'
+parser = argparse.ArgumentParser(description='Introducing main parameters...')
+parser.add_argument('--case',   type=str, default=None)
+parser.add_argument('--dir',    type=str, default=None)
 parser.add_argument('--solver', type=str, default=None)
 
 # DIR = pkg_resources.resource_stream(__name__, CASE)
 # DIR = os.path.join(DIR, CASE)
 DIR = os.path.dirname(openTEPES.__file__)
-SOLVER = "glpk"
-
+SOLVER = 'glpk'
 
 def main():
     args = parser.parse_args()
     if args.dir is None:
-        args.dir = input("Input Dir Name (Default {}): ".format(DIR))
-        if args.dir == "":
+        args.dir = input('Input Dir Name (Default {}): '.format(DIR))
+        if args.dir == '':
             args.dir = DIR
     if args.case is None:
-        args.case = input("Input Case Name (Default {}): ".format(CASE))
-        if args.case == "":
+        args.case = input('Input Case Name (Default {}): '.format(CASE))
+        if args.case == '':
             args.case = CASE
     if args.solver is None:
-        args.solver = input("Input Solver Name (Default {}): ".format(SOLVER))
-        if args.solver == "":
+        args.solver = input('Input Solver Name (Default {}): '.format(SOLVER))
+        if args.solver == '':
             args.solver = SOLVER
     print(args.case)
     print(args.dir)
@@ -122,8 +122,7 @@ def main():
     print(sys.argv)
     print(args)
     openTEPES_run(args.dir, args.case, args.solver)
-    sys.exit("Running Process Finished...")
+    sys.exit('Running Process Finished...')
 
-
-if __name__ == "__main__":
+if __name__ == '__main__':
     main()
