@@ -1,4 +1,5 @@
-# Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - February 7, 2021
+""" Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - April 3, 2021
+"""
 
 import time
 import os
@@ -7,7 +8,7 @@ from   pyomo.opt     import SolverFactory
 from   pyomo.environ import Suffix
 
 def ProblemSolving(DirName, CaseName, SolverName, mTEPES):
-    print('Problem solving             ****')
+    print('Problem solving                        ****')
     _path = os.path.join(DirName, CaseName)
     StartTime = time.time()
 
@@ -23,7 +24,7 @@ def ProblemSolving(DirName, CaseName, SolverName, mTEPES):
         Solver.options['Threads'       ] = int((psutil.cpu_count(logical=True) + psutil.cpu_count(logical=False))/2)
         Solver.options['TimeLimit'     ] =    18000
         Solver.options['IterationLimit'] = 18000000
-    if mTEPES.pIndBinGenInvest*len(mTEPES.gc) + mTEPES.pIndBinNetInvest*len(mTEPES.lc) + mTEPES.pIndBinGenOperat() == 0:
+    if mTEPES.pIndBinGenInvest*len(mTEPES.gc) + mTEPES.pIndBinNetInvest*len(mTEPES.lc) + mTEPES.pIndBinGenOperat() + mTEPES.pIndBinLineCommit() == 0:
         if SolverName == 'gurobi':
             Solver.options['relax_integrality'] =  1                                       # introduced to show results of the dual variables
             Solver.options['Crossover'        ] = -1
@@ -61,6 +62,6 @@ def ProblemSolving(DirName, CaseName, SolverName, mTEPES):
 
     SolvingTime = time.time() - StartTime
     StartTime   = time.time()
-    print('Solving                               ... ', round(SolvingTime), 's')
+    print('Solving                                ... ', round(SolvingTime), 's')
 
-    print('Total system cost [MEUR]                  ', mTEPES.eTotalTCost.expr())
+    print('Total system cost [MEUR]                   ', mTEPES.eTotalTCost.expr())
